@@ -1,5 +1,5 @@
 from gaphor.diagram.presentation import LinePresentation, Named
-from gaphor.diagram.shapes import Box, EditableText, Text, draw_arrow_head
+from gaphor.diagram.shapes import Box, Text, draw_arrow_head
 from gaphor.diagram.support import represents
 from gaphor.SysML import sysml
 from gaphor.UML.modelfactory import stereotypes_str
@@ -15,9 +15,8 @@ class DirectedRelationshipPropertyPathItem(LinePresentation, Named):
         self.shape_middle = Box(
             Text(
                 text=lambda: stereotypes_str(self.subject, (self.relation_type,)),
-                style={"min-width": 0, "min-height": 0},
             ),
-            EditableText(text=lambda: self.subject.name or ""),
+            Text(text=lambda: self.subject.name or ""),
         )
 
         self.draw_head = draw_arrow_head
