@@ -7,7 +7,7 @@ connectors.
 
 from gaphor import UML
 from gaphor.diagram.presentation import LinePresentation, Named
-from gaphor.diagram.shapes import Box, EditableText, Text, draw_arrow_tail
+from gaphor.diagram.shapes import Box, Text, draw_arrow_tail
 from gaphor.diagram.support import represents
 from gaphor.UML.modelfactory import stereotypes_str
 
@@ -27,15 +27,13 @@ class FlowItem(LinePresentation, Named):
             Text(
                 text=lambda: stereotypes_str(self.subject),
             ),
-            EditableText(text=lambda: self.subject.name or ""),
+            Text(text=lambda: self.subject.name or ""),
         )
 
         self.watch("subject[NamedElement].name")
         self.watch("subject.appliedStereotype.classifier.name")
 
-        self.shape_middle = EditableText(
-            text=lambda: self.subject and self.subject.guard or ""
-        )
+        self.shape_middle = Text(text=lambda: self.subject and self.subject.guard or "")
 
         self.watch("subject[ControlFlow].guard")
         self.watch("subject[ObjectFlow].guard")
