@@ -4,9 +4,15 @@ from gi.repository import Gtk
 
 from gaphor import UML
 from gaphor.core import transactional
-from gaphor.diagram.propertypages import PropertyPageBase, PropertyPages, new_builder
+from gaphor.diagram.propertypages import (
+    PropertyPageBase,
+    PropertyPages,
+    new_resource_builder,
+)
 from gaphor.UML.actions.partition import PartitionItem
 
+new_builder = new_resource_builder("gaphor.UML.actions")
+
 
 @PropertyPages.register(PartitionItem)
 class PartitionPropertyPage(PropertyPageBase):
@@ -21,7 +27,10 @@ class PartitionPropertyPage(PropertyPageBase):
 
     def construct(self):
         """Creates the Partition Property Page."""
-        builder = new_builder("partition-editor")
+        builder = new_builder(
+            "partition-editor",
+            signals={"partitions-changed": (self._on_num_partitions_changed,)},
+        )
 
         num_partitions = builder.get_object("num-partitions")
         adjustment = Gtk.Adjustment(
@@ -32,24 +41,23 @@ class PartitionPropertyPage(PropertyPageBase):
             page_increment=5,
         )
         num_partitions.set_adjustment(adjustment)
-        builder.connect_signals(
-            {"partitions-changed": (self._on_num_partitions_changed,)}
-        )
 
         treeview = builder.get_object("partition-treeview")
         treeview.set_model(self.list_store)
 
         renderer_text = Gtk.CellRendererText()
-        column_text = Gtk.TreeViewColumn(title="#", cell_renderer=renderer_text, text=0)
-        treeview.append_column(column_text)
+        treeview.append_column(
+            Gtk.TreeViewColumn(title="#", cell_renderer=renderer_text, text=0)
+        )
 
         renderer_editable_text = Gtk.CellRendererText()
         renderer_editable_text.set_property("editable", True)
 
-        column_editabletext = Gtk.TreeViewColumn(
-            title="Name", cell_renderer=renderer_editable_text, text=1
+        treeview.append_column(
+            Gtk.TreeViewColumn(
+                title="Name", cell_renderer=renderer_editable_text, text=1
+            )
         )
-        treeview.append_column(column_editabletext)
 
         renderer_editable_text.connect("edited", self._on_partition_name_changed)
 
