@@ -2,23 +2,30 @@ from gaphor.UML.classes import (
     classconnect,
     classeseditors,
     classespropertypages,
+    containmentconnect,
     copypaste,
     interfaceconnect,
 )
 from gaphor.UML.classes.association import AssociationItem
 from gaphor.UML.classes.dependency import DependencyItem
 from gaphor.UML.classes.generalization import GeneralizationItem
-from gaphor.UML.classes.implementation import ImplementationItem
+from gaphor.UML.classes.interfacerealization import InterfaceRealizationItem
 from gaphor.UML.classes.interface import Folded, InterfaceItem
 from gaphor.UML.classes.klass import ClassItem
 from gaphor.UML.classes.package import PackageItem
+from gaphor.UML.classes.enumeration import EnumerationItem
+from gaphor.UML.classes.containment import ContainmentItem
+from gaphor.UML.classes.datatype import DataTypeItem
 
 __all__ = [
     "AssociationItem",
+    "ContainmentItem",
+    "DataTypeItem",
     "DependencyItem",
     "GeneralizationItem",
-    "ImplementationItem",
+    "InterfaceRealizationItem",
     "InterfaceItem",
     "ClassItem",
     "PackageItem",
+    "EnumerationItem",
 ]
