@@ -20,17 +20,15 @@ from gaphas.geometry import Rectangle, distance_rectangle_point
 
 from gaphor import UML
 from gaphor.core.modeling.properties import association, attribute
-from gaphor.core.styling import Style
+from gaphor.core.styling import Style, merge_styles
 from gaphor.diagram.presentation import LinePresentation, Named
 from gaphor.diagram.shapes import (
     Box,
-    EditableText,
     Text,
     cairo_state,
-    combined_style,
     draw_default_head,
     draw_default_tail,
-    text_draw_focus_box,
+    stroke,
 )
 from gaphor.diagram.support import represents
 from gaphor.diagram.text import Layout, middle_segment
@@ -55,15 +53,11 @@ class AssociationItem(LinePresentation[UML.Association], Named):
         self._head_end = AssociationEnd(owner=self, end="head")
         self._tail_end = AssociationEnd(owner=self, end="tail")
 
-        # Direction depends on the ends that hold the ownedEnd attributes.
-        self._dir_angle = 0
-        self._dir_pos = 0, 0
-
         self.shape_middle = Box(
             Text(
                 text=lambda: stereotypes_str(self.subject),
             ),
-            EditableText(text=lambda: self.subject.name or ""),
+            Text(text=lambda: self.subject.name or ""),
         )
 
         # For the association ends:
@@ -103,7 +97,7 @@ class AssociationItem(LinePresentation[UML.Association], Named):
             "show_direction"
         )
 
-    show_direction: attribute[bool] = attribute("show_direction", bool, False)
+    show_direction: attribute[int] = attribute("show_direction", int, default=False)
 
     def load(self, name, value):
         # end_head and end_tail were used in an older Gaphor version
@@ -118,8 +112,7 @@ class AssociationItem(LinePresentation[UML.Association], Named):
 
     def postload(self):
         super().postload()
-        self._head_end.set_text()
-        self._tail_end.set_text()
+        self.on_association_end_value()
 
     head_end = property(lambda self: self._head_end)
     tail_end = property(lambda self: self._tail_end)
@@ -137,16 +130,13 @@ class AssociationItem(LinePresentation[UML.Association], Named):
         )
         self.request_update()
 
-    def on_association_end_value(self, event):
+    def update_ends(self):
+        self.on_association_end_value()
+
+    def on_association_end_value(self, event=None):
         """Handle events and update text on association end."""
         for end in (self._head_end, self._tail_end):
             end.set_text()
-        self.request_update()
-
-    def post_update(self, context):
-        """Update the shapes and sub-items of the association."""
-
-        handles = self.handles()
 
         # Update line endings:
         head_subject = self.head_subject
@@ -174,23 +164,11 @@ class AssociationItem(LinePresentation[UML.Association], Named):
                 self.draw_tail = draw_tail_none
             else:
                 self.draw_tail = draw_default_tail
-            if self.show_direction:
-                inverted = self.tail_subject is self.subject.memberEnd[0]
-                pos, angle = get_center_pos(self.handles(), inverted)
-                self._dir_pos = pos
-                self._dir_angle = angle
         else:
             self.draw_head = draw_default_head
             self.draw_tail = draw_default_tail
 
-        # update relationship after self.set calls to avoid circural updates
-        super().post_update(context)
-
-        # Calculate alignment of the head name and multiplicity
-        self._head_end.post_update(context, handles[0].pos, handles[1].pos)
-
-        # Calculate alignment of the tail name and multiplicity
-        self._tail_end.post_update(context, handles[-1].pos, handles[-2].pos)
+        self.request_update()
 
     def point(self, x, y):
         """Returns the distance from the Association to the (mouse) cursor."""
@@ -200,12 +178,25 @@ class AssociationItem(LinePresentation[UML.Association], Named):
 
     def draw(self, context):
         super().draw(context)
+
+        handles = self.handles()
+
+        # Calculate alignment of the head name and multiplicity
+        self._head_end.update_position(context, handles[0].pos, handles[1].pos)
+
+        # Calculate alignment of the tail name and multiplicity
+        self._tail_end.update_position(context, handles[-1].pos, handles[-2].pos)
+
         self._head_end.draw(context)
         self._tail_end.draw(context)
         if self.show_direction:
+            inverted = (
+                self.tail_subject and self.tail_subject is self.subject.memberEnd[0]
+            )
+            pos, angle = get_center_pos(handles, inverted)
             with cairo_state(context.cairo) as cr:
-                cr.translate(*self._dir_pos)
-                cr.rotate(self._dir_angle)
+                cr.translate(*pos)
+                cr.rotate(angle)
                 cr.move_to(0, 0)
                 cr.line_to(6, 5)
                 cr.line_to(0, 10)
@@ -271,8 +262,10 @@ def draw_tail_composite(context):
     at association tail."""
     cr = context.cairo
     cr.line_to(20, 0)
+    stroke(context)
     _draw_diamond(cr)
     cr.fill_preserve()
+    stroke(context)
 
 
 def draw_head_shared(context):
@@ -335,7 +328,7 @@ class AssociationEnd:
         self._name_layout = Layout("")
         self._mult_layout = Layout("")
 
-        self._inline_style: Style = {"font-size": 10}
+        self._inline_style: Style = {"font-size": "x-small"}
 
     name_bounds = property(lambda s: s._name_bounds)
 
@@ -376,13 +369,13 @@ class AssociationEnd:
     def get_mult(self):
         return self._mult
 
-    def post_update(self, context, p1, p2):
+    def update_position(self, context, p1, p2):
         """Update label placement for association's name and multiplicity
         label.
 
         p1 is the line end and p2 is the last but one point of the line.
         """
-        style = combined_style(context.style, self._inline_style)
+        style = merge_styles(context.style, self._inline_style)
         ofs = 5
 
         dx = float(p2[0]) - float(p1[0])
@@ -486,6 +479,3 @@ class AssociationEnd:
         self._name_layout.show_layout(cr)
         cr.move_to(self._mult_bounds.x, self._mult_bounds.y)
         self._mult_layout.show_layout(cr)
-
-        for b in (self._name_bounds, self._mult_bounds):
-            text_draw_focus_box(context, b.x, b.y, b.width, b.height)
