@@ -2,7 +2,7 @@
 
 from gaphor import UML
 from gaphor.diagram.presentation import ElementPresentation, Named, from_package_str
-from gaphor.diagram.shapes import Box, EditableText, Text, cairo_state, stroke
+from gaphor.diagram.shapes import Box, Text, cairo_state, stroke
 from gaphor.diagram.support import represents
 from gaphor.diagram.text import FontWeight
 from gaphor.UML.modelfactory import stereotypes_str
@@ -12,7 +12,7 @@ from gaphor.UML.modelfactory import stereotypes_str
 @represents(UML.Profile)
 class PackageItem(ElementPresentation, Named):
     def __init__(self, diagram, id=None):
-        super().__init__(diagram, id)
+        super().__init__(diagram, id, width=70, height=70)
 
         self.shape = Box(
             Text(
@@ -21,15 +21,15 @@ class PackageItem(ElementPresentation, Named):
                     isinstance(self.subject, UML.Profile) and ("profile",) or (),
                 ),
             ),
-            EditableText(
+            Text(
                 text=lambda: self.subject and self.subject.name or "",
                 style={"font-weight": FontWeight.BOLD},
             ),
             Text(
                 text=lambda: from_package_str(self),
-                style={"font-size": 10, "min-width": 0, "min-height": 0},
+                style={"font-size": "x-small"},
             ),
-            style={"min-width": 50, "min-height": 70, "padding": (25, 10, 5, 10)},
+            style={"padding": (24, 12, 4, 12)},
             draw=draw_package,
         )
 
@@ -52,4 +52,4 @@ def draw_package(box, context, bounding_box):
         cr.line_to(w, h)
         cr.line_to(w, y)
         cr.line_to(o, y)
-        stroke(context, highlight=True)
+        stroke(context)
