@@ -7,7 +7,7 @@ Functions collected in this module allow to
 """
 
 import itertools
-from typing import Iterable, Sequence
+from typing import Iterable, Optional, Sequence
 
 from gaphor.UML.uml import (
     Association,
@@ -22,11 +22,11 @@ from gaphor.UML.uml import (
     Extension,
     ExtensionEnd,
     Generalization,
-    Implementation,
     InstanceSpecification,
     Interface,
     Message,
     MessageOccurrenceSpecification,
+    Package,
     Port,
     Property,
     Realization,
@@ -224,17 +224,6 @@ def create_generalization(general, specific):
     return gen
 
 
-def create_implementation(contract, implementatingClassifier):
-    assert (
-        contract.model is implementatingClassifier.model
-    ), "Contract and Implementing classifier are from different models"
-    model = contract.model
-    impl = model.create(Implementation)
-    impl.contract = contract
-    impl.implementatingClassifier = implementatingClassifier
-    return impl
-
-
 def create_association(type_a, type_b):
     """Create an association between two items."""
     assert type_a.model is type_b.model, "Head and Tail end are from different models"
@@ -330,6 +319,7 @@ def set_navigability(assoc, end, nav):
         assoc.ownedEnd.remove(end)
     if end in assoc.navigableOwnedEnd:
         assoc.navigableOwnedEnd.remove(end)
+
     assert end not in assoc.ownedEnd
     assert end not in assoc.navigableOwnedEnd
 
@@ -360,7 +350,7 @@ def dependency_type(client, supplier):
     # test interface first as it is a classifier
     if isinstance(supplier, Interface):
         dt = Usage
-    elif isinstance(client, Component) and isinstance(supplier, Classifier):
+    elif isinstance(supplier, Component) and isinstance(client, Classifier):
         dt = Realization
 
     return dt
@@ -394,6 +384,12 @@ def clone_message(msg, inverted=False):
     return message
 
 
+def owner_package(element: Optional[Element]) -> Optional[Package]:
+    if element is None or isinstance(element, Package):
+        return element
+    return owner_package(element.owner)
+
+
 def swap_element(element, new_class):
     """A "trick" to swap the element type.
 
