@@ -2,7 +2,7 @@
 
 from gaphor import UML
 from gaphor.diagram.presentation import LinePresentation, Named
-from gaphor.diagram.shapes import Box, EditableText, Text, draw_arrow_tail
+from gaphor.diagram.shapes import Box, Text, draw_arrow_tail
 from gaphor.diagram.support import represents
 from gaphor.UML.modelfactory import stereotypes_str
 
@@ -18,13 +18,13 @@ class TransitionItem(LinePresentation[UML.Transition], Named):
             Text(
                 text=lambda: stereotypes_str(self.subject),
             ),
-            EditableText(text=lambda: self.subject.name or ""),
+            Text(text=lambda: self.subject.name or ""),
         )
 
         self.watch("subject[NamedElement].name")
         self.watch("subject.appliedStereotype.classifier.name")
 
-        self.shape_middle = EditableText(
+        self.shape_middle = Text(
             text=lambda: self.subject
             and self.subject.guard
             and self.subject.guard.specification
