@@ -44,6 +44,9 @@ rest_subpat = r"\s*(,\s*(?P<rest>.*))?"
 # Direction of a parameter (optional, default in) ::= 'in' | 'out' | 'inout'
 dir_subpat = r"\s*((?P<dir>in|out|inout)\s)?"
 
+# A note ::= '#' text
+note_subpat = r"\s*(#\s*(?P<note>.*))?"
+
 # Some trailing garbage => no valid syntax...
 garbage_subpat = r"\s*(?P<garbage>.*)"
 
@@ -63,6 +66,7 @@ attribute_pat = compile(
     + mult_subpat
     + default_subpat
     + tags_subpat
+    + note_subpat
     + garbage_subpat
 )
 
@@ -78,6 +82,7 @@ association_end_name_pat = compile(
     + mult_subpat
     + ")?"
     + tags_subpat
+    + note_subpat
     + garbage_subpat
 )
 
@@ -95,6 +100,7 @@ operation_pat = compile(
     + type_subpat
     + mult_subpat
     + tags_subpat
+    + note_subpat
     + garbage_subpat
 )
 
@@ -151,6 +157,8 @@ def parse_attribute(el: uml.Property, s: str) -> None:
             el.upperValue = None
         if el.defaultValue:
             el.defaultValue = None
+        if el.note:
+            el.note = None
     else:
         g = m.group
         _set_visibility(el, g("vis"))
@@ -160,6 +168,7 @@ def parse_attribute(el: uml.Property, s: str) -> None:
         el.lowerValue = g("mult_l")
         el.upperValue = g("mult_u")
         el.defaultValue = g("default")
+        el.note = g("note")
 
 
 def parse_association_end(el: uml.Property, s: str) -> None:
@@ -192,10 +201,12 @@ def parse_association_end(el: uml.Property, s: str) -> None:
             el.name = s
             del el.visibility
             del el.isDerived
+            del el.note
         else:
             _set_visibility(el, g("vis"))
             el.isDerived = g("derived") and True or False
             el.name = g("name")
+            el.note = g("note")
             # Optionally, the multiplicity and tagged values may be defined:
             if g("mult_l"):
                 el.lowerValue = g("mult_l")
@@ -225,20 +236,28 @@ def parse_operation(el: uml.Operation, s: str) -> None:
     if not m or m.group("garbage"):
         el.name = s
         del el.visibility
-        list(map(uml.Parameter.unlink, list(el.returnResult)))
-        list(map(uml.Parameter.unlink, list(el.formalParameter)))
+        for param in list(el.ownedParameter):
+            param.unlink()
     else:
         g = m.group
         create = el.model.create
         _set_visibility(el, g("vis"))
         el.name = g("name")
-        if not el.returnResult:
-            el.returnResult = create(uml.Parameter)
-        p = el.returnResult[0]
-        p.direction = "return"
-        p.typeValue = g("type")
-        p.lowerValue = g("mult_l")
-        p.upperValue = g("mult_u")
+        el.note = g("note")
+
+        defined_params = set()
+        if g("type"):
+            returnParameters = [p for p in el.ownedParameter if p.direction == "return"]
+            if returnParameters:
+                p = returnParameters[0]
+            else:
+                p = create(uml.Parameter)
+                el.ownedParameter = p
+                p.direction = "return"
+            p.typeValue = g("type")
+            p.lowerValue = g("mult_l")
+            p.upperValue = g("mult_u")
+            defined_params.add(p)
 
         pindex = 0
         params = g("params")
@@ -248,7 +267,9 @@ def parse_operation(el: uml.Operation, s: str) -> None:
                 break
             g = m.group
             try:
-                p = el.formalParameter[pindex]
+                while el.ownedParameter[pindex] in defined_params:
+                    pindex += 1
+                p = el.ownedParameter[pindex]
             except IndexError:
                 p = create(uml.Parameter)
             p.direction = g("dir") or "in"
@@ -257,15 +278,15 @@ def parse_operation(el: uml.Operation, s: str) -> None:
             p.lowerValue = g("mult_l")
             p.upperValue = g("mult_u")
             p.defaultValue = g("default")
-            el.formalParameter = p
-
+            el.ownedParameter = p
+            defined_params.add(p)
             # Do the next parameter:
             params = g("rest")
-            pindex += 1
 
         # Remove remaining parameters:
-        for fp in el.formalParameter[pindex:]:
-            fp.unlink()
+        for op in list(el.ownedParameter):
+            if op not in defined_params:
+                op.unlink()
 
 
 def parse_lifeline(el: uml.Lifeline, s: str) -> None:
@@ -288,7 +309,6 @@ def parse_lifeline(el: uml.Lifeline, s: str) -> None:
 
 
 def render_lifeline(el: uml.Lifeline) -> str:
-    """"""
     return el.name or ""
 
 
