@@ -2,7 +2,7 @@
 
 from gaphor import UML
 from gaphor.diagram.presentation import LinePresentation, Named
-from gaphor.diagram.shapes import Box, EditableText, Text, draw_arrow_head
+from gaphor.diagram.shapes import Box, Text, draw_arrow_head
 from gaphor.diagram.support import represents
 from gaphor.UML.modelfactory import stereotypes_str
 
@@ -16,7 +16,7 @@ class ExtendItem(LinePresentation, Named):
 
         self.shape_middle = Box(
             Text(text=lambda: stereotypes_str(self.subject, ("extend",))),
-            EditableText(text=lambda: self.subject.name or ""),
+            Text(text=lambda: self.subject.name or ""),
         )
         self.watch("subject.appliedStereotype.classifier.name").watch(
             "subject[NamedElement].name"
