@@ -4,9 +4,14 @@ from gaphor.diagram.inlineeditors import InlineEditor
 
 
 def text_edit_tools(view):
-    key_tool = Gtk.EventControllerKey.new(view)
+    if Gtk.get_major_version() == 3:
+        key_tool = Gtk.EventControllerKey.new(view)
+        click_tool = Gtk.GestureMultiPress.new(view)
+    else:
+        key_tool = Gtk.EventControllerKey.new()
+        click_tool = Gtk.GestureClick.new()
+
     key_tool.connect("key-pressed", on_key_pressed)
-    click_tool = Gtk.GestureMultiPress.new(view)
     click_tool.connect("released", on_double_click)
     return key_tool, click_tool
 
