@@ -1,19 +1,17 @@
 from __future__ import annotations
 
 from functools import singledispatch
-from typing import Optional, Tuple
 
 from gaphas import Item
 from gaphas.geometry import Rectangle
 from gi.repository import Gdk, Gtk
 
 from gaphor.core import transactional
-from gaphor.diagram import shapes as _shapes
-from gaphor.diagram.presentation import ElementPresentation, LinePresentation, Named
+from gaphor.diagram.presentation import LinePresentation, Named
 
 
 @singledispatch
-def InlineEditor(item: Item, view, pos: Optional[Tuple[int, int]] = None) -> bool:
+def InlineEditor(item: Item, view, pos: tuple[int, int] | None = None) -> bool:
     """Show a small editor popup in the diagram. Makes for easy editing without
     resorting to the Element editor.
 
@@ -36,8 +34,13 @@ def named_item_inline_editor(item, view, pos=None) -> bool:
     if not subject:
         return False
 
-    box = editable_text_box(view, view.selection.hovered_item)
-    if not box:
+    if isinstance(item, LinePresentation):
+        box = item.middle_shape_size
+        i2v = view.get_matrix_i2v(item)
+        x, y = i2v.transform_point(box.x, box.y)
+        w, h = i2v.transform_distance(box.width, box.height)
+        box = Rectangle(x, y, w, h)
+    else:
         box = view.get_item_bounding_box(view.selection.hovered_item)
     name = subject.name or ""
     entry = popup_entry(name, update_text)
@@ -61,8 +64,11 @@ def popup_entry(text, update_text, done=None):
 
 def show_popover(widget, view, box, escape=None):
     popover = Gtk.Popover.new()
-    popover.add(widget)
-    popover.set_relative_to(view)
+    if Gtk.get_major_version() == 3:
+        popover.add(widget)
+        popover.set_relative_to(view)
+    else:
+        popover.set_child(widget)
     gdk_rect = Gdk.Rectangle()
     gdk_rect.x = box.x
     gdk_rect.y = box.y
@@ -70,36 +76,26 @@ def show_popover(widget, view, box, escape=None):
     gdk_rect.height = box.height
     popover.set_pointing_to(gdk_rect)
 
-    def on_escape(popover, event):
-        if event.keyval == Gdk.KEY_Return and not event.get_state() & (
+    def on_escape(popover, keyval, keycode, state):
+        if keyval == Gdk.KEY_Return and not state & (
             Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK
         ):
             popover.popdown()
             return True
-        elif event.keyval == Gdk.KEY_Escape and escape:
+        elif keyval == Gdk.KEY_Escape and escape:
             escape()
 
-    popover.connect("key-press-event", on_escape)
-    popover.popup()
+    if Gtk.get_major_version() == 3:
+
+        def on_escape3(popover, event):
+            return on_escape(popover, event.keyval, 0, event.get_state())
+
+        popover.connect("key-press-event", on_escape3)
+        popover.popup()
+    else:
+        controller = Gtk.EventControllerKey.new()
+        popover.add_controller(controller)
+        controller.connect("key-pressed", on_escape)
+        popover.present()
+
     return popover
-
-
-def editable_text_box(view, item):
-    def find(*shapes):
-        for shape in shapes:
-            if isinstance(shape, (_shapes.Box, _shapes.IconBox)):
-                box = find(*shape.children)
-                if box:
-                    return box
-            elif isinstance(shape, _shapes.EditableText):
-                box = shape.bounding_box
-                i2v = view.get_matrix_i2v(item)
-                x, y = i2v.transform_point(box.x, box.y)
-                w, h = i2v.transform_distance(box.width, box.height)
-                # Add a little bit of padding, 'cause that makes it look so good
-                return Rectangle(x - 6, y - 6, w + 12, h + 12)
-
-    if isinstance(item, ElementPresentation):
-        return find(item.shape)
-    elif isinstance(item, LinePresentation):
-        return find(item.shape_middle, item.shape_head, item.shape_tail)
