@@ -139,10 +139,11 @@ class Transaction:
         """
 
         if exc_type:
-            log.error(
-                "Transaction terminated due to an exception, performing a rollback",
-                exc_info=True,
-            )
+            if not self._need_rollback:
+                log.error(
+                    "Transaction terminated due to an exception, performing a rollback",
+                    exc_info=True,
+                )
             try:
                 self.rollback()
             except Exception:
