@@ -1,6 +1,5 @@
 """The element editor is a utility window used for editing elements."""
 
-import importlib.resources
 import logging
 from typing import Optional
 
@@ -15,19 +14,14 @@ from gaphor.core.modeling.event import (
     ElementCreated,
     ModelReady,
 )
-from gaphor.diagram.propertypages import PropertyPages
+from gaphor.diagram.propertypages import PropertyPages, new_resource_builder
 from gaphor.ui.abc import UIComponent
 from gaphor.ui.event import DiagramSelectionChanged
 
 log = logging.getLogger(__name__)
 
 
-def new_builder(*object_ids):
-    builder = Gtk.Builder()
-    builder.set_translation_domain("gaphor")
-    with importlib.resources.path("gaphor.ui", "elementeditor.glade") as glade_file:
-        builder.add_objects_from_file(str(glade_file), object_ids)
-    return builder
+new_builder = new_resource_builder("gaphor.ui", "elementeditor")
 
 
 class DelayedFunction:
@@ -70,6 +64,7 @@ class ElementEditor(UIComponent, ActionProvider):
         builder = new_builder("elementeditor", "style-sheet-buffer")
 
         self.revealer = builder.get_object("elementeditor")
+        self.revealer.set_reveal_child(self.properties.get("show-editors", True))
         self.editor_stack = builder.get_object("editor-stack")
 
         self.editors.open(builder)
@@ -89,9 +84,14 @@ class ElementEditor(UIComponent, ActionProvider):
 
         return True
 
-    @action(name="show-editors", shortcut="<Primary>e", state=True)
+    @action(
+        name="show-editors",
+        shortcut="<Primary>e",
+        state=lambda self: self.properties.get("show-editors", True),
+    )
     def toggle_editor_visibility(self, active):
         self.revealer.set_reveal_child(active)
+        self.properties.set("show-editors", active)
 
     @action(name="show-settings", state=False)
     def toggle_editor_settings(self, active):
@@ -158,7 +158,10 @@ class EditorStack:
                 elif isinstance(page, Gtk.Expander):
                     page.set_expanded(self._expanded_pages.get(name, True))
                     page.connect_after("activate", self.on_expand, name)
-                self.vbox.pack_start(page, False, True, 0)
+                if Gtk.get_major_version() == 3:
+                    self.vbox.pack_start(page, False, True, 0)
+                else:
+                    self.vbox.append(page)
             except Exception:
                 log.error(
                     "Could not construct property page for " + name, exc_info=True
@@ -167,8 +170,14 @@ class EditorStack:
     def clear_pages(self):
         """Remove all tabs from the notebook."""
         assert self.vbox
-        for page in self.vbox.get_children():
-            page.destroy()
+        if Gtk.get_major_version() == 3:
+            for page in self.vbox.get_children():
+                page.destroy()
+        else:
+            page = self.vbox.get_first_child()
+            while page:
+                self.vbox.remove(page)
+                page = self.vbox.get_first_child()
 
     def on_expand(self, widget, name):
         self._expanded_pages[name] = widget.get_expanded()
@@ -181,7 +190,12 @@ class EditorStack:
         """
         assert self.vbox
         item = event and event.focused_item or focused_item
-        if item is self._current_item and self.vbox.get_children():
+        children = (
+            self.vbox.get_children()
+            if Gtk.get_major_version() == 3
+            else self.vbox.get_first_child()
+        )
+        if item is self._current_item and children:
             return
 
         self._current_item = item
@@ -196,12 +210,15 @@ class EditorStack:
         assert self.vbox
         builder = new_builder("no-item-selected")
 
-        self.vbox.pack_start(
-            child=builder.get_object("no-item-selected"),
-            expand=False,
-            fill=True,
-            padding=0,
-        )
+        if Gtk.get_major_version() == 3:
+            self.vbox.pack_start(
+                child=builder.get_object("no-item-selected"),
+                expand=False,
+                fill=True,
+                padding=0,
+            )
+        else:
+            self.vbox.append(builder.get_object("no-item-selected"))
 
         tips = builder.get_object("tips")
 
@@ -248,6 +265,7 @@ class SettingsStack:
         self.event_manager.subscribe(self._style_sheet_created)
         self.event_manager.subscribe(self._style_sheet_changed)
         self.style_sheet_buffer.connect("changed", self.on_style_sheet_changed)
+        self.update_text()
 
     def close(self):
         self.event_manager.unsubscribe(self._model_ready)
