@@ -8,23 +8,24 @@ classifiers are shown here.
 from __future__ import annotations
 
 import logging
-from typing import TYPE_CHECKING, Optional, Set
+from typing import TYPE_CHECKING
 
 from gi.repository import Gdk, Gio, GLib, Gtk
 
 from gaphor import UML
+from gaphor.abc import ActionProvider
 from gaphor.core import action, event_handler, gettext, transactional
-from gaphor.core.format import format
-from gaphor.core.modeling import Diagram, Presentation
+from gaphor.core.modeling import Diagram, Element, Presentation
 from gaphor.ui.abc import UIComponent
 from gaphor.ui.actiongroup import create_action_group
 from gaphor.ui.event import DiagramOpened, DiagramSelectionChanged
 from gaphor.ui.namespacemodel import (
     RELATIONSHIPS,
     NamespaceModel,
+    NamespaceModelElementDropped,
     NamespaceModelRefreshed,
 )
-from gaphor.ui.namespaceview import NamespaceView
+from gaphor.ui.namespaceview import namespace_view
 
 if TYPE_CHECKING:
     from gaphor.core.eventmanager import EventManager
@@ -34,7 +35,7 @@ if TYPE_CHECKING:
 log = logging.getLogger(__name__)
 
 
-def popup_model(view):
+def popup_model(element):
     model = Gio.Menu.new()
 
     part = Gio.Menu.new()
@@ -43,7 +44,7 @@ def popup_model(view):
     model.append_section(None, part)
 
     part = Gio.Menu.new()
-    part.append(gettext("New _Diagram"), "tree-view.create-diagram")
+    part.append(gettext("New _Diagram"), "win.create-diagram")
     part.append(gettext("New _Package"), "tree-view.create-package")
     model.append_section(None, part)
 
@@ -51,7 +52,8 @@ def popup_model(view):
     part.append(gettext("De_lete"), "tree-view.delete")
     model.append_section(None, part)
 
-    element = view.get_selected_element()
+    if not element:
+        return model
 
     part = Gio.Menu.new()
     for presentation in element.presentation:
@@ -73,70 +75,60 @@ def popup_model(view):
     return model
 
 
-class Namespace(UIComponent):
+class Namespace(UIComponent, ActionProvider):
     def __init__(self, event_manager: EventManager, element_factory: ElementFactory):
         self.event_manager = event_manager
         self.element_factory = element_factory
 
-        self.model: Optional[NamespaceModel] = None
-        self.view: Optional[NamespaceView] = None
-        self.ctrl: Set[Gtk.EventController] = set()
+        self.model: NamespaceModel | None = None
+        self.view: Gtk.TreeView | None = None
+        self.scrolled_window: Gtk.ScrolledWindow | None = None
+        self.ctrl: set[Gtk.EventController] = set()
 
     def open(self):
         self.model = NamespaceModel(self.event_manager, self.element_factory)
+        view = namespace_view(self.model)
         self.event_manager.subscribe(self._on_model_refreshed)
+        self.event_manager.subscribe(self._on_model_dropped)
         self.event_manager.subscribe(self._on_diagram_selection_changed)
 
-        sorted_model = self.model.sorted()
-
-        def search_func(model, column, key, rowiter):
-            # Note that this function returns `False` for a match!
-            assert column == 0
-            row = model[rowiter]
-            matched = False
-
-            # Search in child rows.  If any element in the underlying
-            # tree matches, it will expand.
-            for inner in row.iterchildren():
-                if not search_func(model, column, key, inner.iter):
-                    view.expand_to_path(row.path)
-                    matched = True
-
-            element = list(row)[column]
-            s = format(element)
-            if s and key.lower() in s.lower():
-                matched = True
-            elif not matched:
-                view.collapse_row(row.path)
-
-            return not matched  # False means match found!
-
-        view = NamespaceView(sorted_model, self.element_factory)
-        view.set_search_equal_func(search_func)
-
         scrolled_window = Gtk.ScrolledWindow()
         scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        scrolled_window.add(view)
+        if Gtk.get_major_version() == 3:
+            scrolled_window.add(view)
+        else:
+            scrolled_window.set_child(view)
         scrolled_window.show()
         view.show()
 
-        scrolled_window.insert_action_group(
-            "tree-view", create_action_group(self, "tree-view")[0]
-        )
         view.connect("row-activated", self._on_view_row_activated)
         view.connect_after("cursor-changed", self._on_view_cursor_changed)
         view.connect("destroy", self._on_view_destroyed)
 
-        ctrl = Gtk.GestureMultiPress.new(view)
-        ctrl.set_button(Gdk.BUTTON_SECONDARY)
-        ctrl.connect("pressed", self._on_show_popup)
-        self.ctrl.add(ctrl)
+        action_group, shortcuts = create_action_group(self, "tree-view")
+        scrolled_window.insert_action_group("tree-view", action_group)
 
-        ctrl = Gtk.EventControllerKey.new(view)
-        ctrl.connect("key-pressed", self._on_edit_pressed)
-        self.ctrl.add(ctrl)
+        if Gtk.get_major_version() == 3:
+            ctrl = Gtk.GestureMultiPress.new(view)
+            ctrl.set_button(Gdk.BUTTON_SECONDARY)
+            ctrl.connect("pressed", self._on_show_popup)
+            self.ctrl.add(ctrl)
+
+            ctrl = Gtk.EventControllerKey.new(view)
+            ctrl.connect("key-pressed", self._on_edit_pressed)
+            self.ctrl.add(ctrl)
+        else:
+            ctrl = Gtk.ShortcutController.new_for_model(shortcuts)
+            ctrl.set_scope(Gtk.ShortcutScope.LOCAL)
+            view.add_controller(ctrl)
+
+            ctrl = Gtk.GestureClick.new()
+            ctrl.set_button(Gdk.BUTTON_SECONDARY)
+            ctrl.connect("pressed", self._on_show_popup)
+            view.add_controller(ctrl)
 
         self.view = view
+        self.scrolled_window = scrolled_window
         self.model.refresh()
 
         return scrolled_window
@@ -144,12 +136,17 @@ class Namespace(UIComponent):
     def close(self):
         self.ctrl.clear()
         if self.view:
-            self.view.destroy()
+            if Gtk.get_major_version() == 3:
+                self.view.destroy()
+            elif self.scrolled_window:
+                self.scrolled_window.unparent()
             self.view = None
+            self.scrolled_window = None
         if self.model:
             self.model.shutdown()
             self.model = None
         self.event_manager.unsubscribe(self._on_model_refreshed)
+        self.event_manager.unsubscribe(self._on_model_dropped)
         self.event_manager.unsubscribe(self._on_diagram_selection_changed)
 
     @event_handler(NamespaceModelRefreshed)
@@ -159,6 +156,11 @@ class Namespace(UIComponent):
             self.view.expand_row(path=Gtk.TreePath.new_first(), open_all=False)
             self._on_view_cursor_changed(self.view)
 
+    @event_handler(NamespaceModelElementDropped)
+    def _on_model_dropped(self, event):
+        element = event.element
+        self.select_element(element)
+
     @event_handler(DiagramSelectionChanged)
     def _on_diagram_selection_changed(self, event):
         focused_item = event.focused_item
@@ -166,9 +168,19 @@ class Namespace(UIComponent):
             self.select_element(focused_item.subject)
 
     def _on_show_popup(self, ctrl, n_press, x, y):
-        menu = Gtk.Menu.new_from_model(popup_model(self.view))
-        menu.attach_to_widget(self.view, None)
-        menu.popup_at_pointer(None)
+        if Gtk.get_major_version() == 3:
+            menu = Gtk.Menu.new_from_model(popup_model(self.get_selected_element()))
+            menu.attach_to_widget(self.view, None)
+            menu.popup_at_pointer(None)
+        else:
+            menu = Gtk.PopoverMenu.new_from_model(
+                popup_model(self.get_selected_element())
+            )
+            menu.set_pointing_to(Gdk.Rectangle(x, y, 1, 1))
+            menu.set_offset(x, y)
+            menu.set_has_arrow(False)
+            menu.set_parent(self.view)
+            menu.popup()
 
     def _on_edit_pressed(self, ctrl, keyval, keycode, state):
         if keyval == Gdk.KEY_F2:
@@ -178,33 +190,36 @@ class Namespace(UIComponent):
 
     def _on_view_row_activated(self, view, path, column):
         """Double click on an element in the tree view."""
-        view.get_action_group("tree-view").lookup_action("open").activate()
+        if Gtk.get_major_version() == 3:
+            view.get_action_group("tree-view").lookup_action("open").activate()
+        else:
+            view.activate_action("tree-view.open", None)
 
     def _on_view_cursor_changed(self, view):
         """Another row is selected, toggle action sensitivity."""
-        element = view.get_selected_element()
-        action_group = view.get_action_group("tree-view")
+        element = self.get_selected_element()
+        if Gtk.get_major_version() == 3:
+            action_group = view.get_action_group("tree-view")
 
-        action_group.lookup_action("open").set_enabled(isinstance(element, Diagram))
-        action_group.lookup_action("create-diagram").set_enabled(
-            isinstance(element, UML.Package)
-            or (element and isinstance(element.owner, UML.Package))
-        )
-        action_group.lookup_action("create-package").set_enabled(
-            isinstance(element, UML.Package)
-        )
-        action_group.lookup_action("delete").set_enabled(
-            isinstance(element, Diagram)
-            or (isinstance(element, UML.Package) and not element.presentation)
-        )
-        action_group.lookup_action("rename").set_enabled(
-            isinstance(element, UML.NamedElement)
-        )
+            action_group.lookup_action("open").set_enabled(isinstance(element, Diagram))
+            action_group.lookup_action("create-package").set_enabled(
+                isinstance(element, UML.Package)
+            )
+            action_group.lookup_action("delete").set_enabled(
+                isinstance(element, Diagram)
+                or (isinstance(element, UML.Package) and not element.presentation)
+            )
+            action_group.lookup_action("rename").set_enabled(
+                isinstance(element, UML.NamedElement)
+            )
+        else:
+            # TODO: GTK4 - enable/disable actions based on view state
+            pass
 
     def _on_view_destroyed(self, widget):
         self.close()
 
-    def select_element(self, element):
+    def select_element(self, element: Element) -> None:
         """Select an element from the Namespace view.
 
         The element is selected. After this an action may be executed,
@@ -215,12 +230,10 @@ class Namespace(UIComponent):
         assert self.view
 
         model = self.view.get_model()
-        child_iter = self.model.iter_for_element(element)
-        if not child_iter:
+        tree_iter = self.model.iter_for_element(element)
+        if not tree_iter:
             return
 
-        ok, tree_iter = model.convert_child_iter_to_iter(child_iter)
-        assert ok, "Could not convert model iterator to view"
         path = model.get_path(tree_iter)
         path_indices = path.get_indices()
 
@@ -234,10 +247,17 @@ class Namespace(UIComponent):
         self.view.scroll_to_cell(path, None, False, 0, 0)
         self._on_view_cursor_changed(self.view)
 
+    def get_selected_element(self) -> Element | None:
+        assert self.view
+        selection = self.view.get_selection()
+        model, iter = selection.get_selected()
+        if not iter:
+            return None
+        return model.get_value(iter, 0)  # type: ignore[no-any-return]
+
     @action(name="tree-view.open")
     def tree_view_open_selected(self):
-        assert self.view
-        element = self.view.get_selected_element()
+        element = self.get_selected_element()
         if isinstance(element, Diagram):
             self.event_manager.handle(DiagramOpened(element))
         else:
@@ -252,7 +272,7 @@ class Namespace(UIComponent):
     def tree_view_rename_selected(self):
         assert self.view
         view = self.view
-        element = view.get_selected_element()
+        element = self.get_selected_element()
         if element not in (None, RELATIONSHIPS):
             selection = view.get_selection()
             model, iter = selection.get_selected()
@@ -263,18 +283,17 @@ class Namespace(UIComponent):
             view.set_cursor(path, column, True)
             cell.set_property("editable", 0)
 
-    @action(name="tree-view.create-diagram")
+    @action(name="win.create-diagram")
     @transactional
     def tree_view_create_diagram(self):
         assert self.view
-        element = self.view.get_selected_element()
-        assert element
-        while not isinstance(element, UML.Package):
-            element = element.owner
+        element = self.get_selected_element()
         diagram = self.element_factory.create(Diagram)
-        diagram.package = element
-
-        diagram.name = f"{element.name} diagram" if element else "New diagram"
+        if isinstance(element, UML.NamedElement):
+            diagram.element = element
+            diagram.name = f"{element.name} diagram"
+        else:
+            diagram.name = "New diagram"
         self.select_element(diagram)
         self.event_manager.handle(DiagramOpened(diagram))
         self.tree_view_rename_selected()
@@ -282,8 +301,7 @@ class Namespace(UIComponent):
     @action(name="tree-view.create-package")
     @transactional
     def tree_view_create_package(self):
-        assert self.view
-        element = self.view.get_selected_element()
+        element = self.get_selected_element()
         assert isinstance(element, UML.Package)
         package = self.element_factory.create(UML.Package)
         package.package = element
@@ -295,8 +313,7 @@ class Namespace(UIComponent):
     @action(name="tree-view.delete")
     @transactional
     def tree_view_delete(self):
-        assert self.view
-        element = self.view.get_selected_element()
+        element = self.get_selected_element()
         if isinstance(element, UML.Package):
             element.unlink()
         elif isinstance(element, Diagram):
